@@ -6,17 +6,17 @@
  * access control decisions recently used, and reduce number of kernel
  * invocations to avoid unnecessary performance hit.
  *
- * Copyright (c) 2011-2015, PostgreSQL Global Development Group
+ * Copyright (c) 2011-2019, PostgreSQL Global Development Group
  *
  * -------------------------------------------------------------------------
  */
 #include "postgres.h"
 
-#include "access/hash.h"
 #include "catalog/pg_proc.h"
 #include "commands/seclabel.h"
 #include "storage/ipc.h"
 #include "utils/guc.h"
+#include "utils/hashutils.h"
 #include "utils/memutils.h"
 
 #include "sepgsql.h"
@@ -45,7 +45,7 @@ typedef struct
 	/* true, if tcontext is valid */
 	char	   *ncontext;		/* temporary scontext on execution of trusted
 								 * procedure, or NULL elsewhere */
-}	avc_cache;
+}			avc_cache;
 
 /*
  * Declaration of static variables
@@ -182,7 +182,7 @@ sepgsql_avc_unlabeled(void)
 		if (security_get_initial_context_raw("unlabeled", &unlabeled) < 0)
 			ereport(ERROR,
 					(errcode(ERRCODE_INTERNAL_ERROR),
-			   errmsg("SELinux: failed to get initial security label: %m")));
+					 errmsg("SELinux: failed to get initial security label: %m")));
 		PG_TRY();
 		{
 			avc_unlabeled = MemoryContextStrdup(avc_mem_cxt, unlabeled);
@@ -407,7 +407,7 @@ sepgsql_avc_check_perms_label(const char *tcontext,
 		audit_name != SEPGSQL_AVC_NOAUDIT &&
 		sepgsql_get_mode() != SEPGSQL_MODE_INTERNAL)
 	{
-		sepgsql_audit_log(!!denied,
+		sepgsql_audit_log(denied != 0,
 						  cache->scontext,
 						  cache->tcontext_is_valid ?
 						  cache->tcontext : sepgsql_avc_unlabeled(),
@@ -498,13 +498,11 @@ sepgsql_avc_init(void)
 	int			rc;
 
 	/*
-	 * All the avc stuff shall be allocated on avc_mem_cxt
+	 * All the avc stuff shall be allocated in avc_mem_cxt
 	 */
 	avc_mem_cxt = AllocSetContextCreate(TopMemoryContext,
 										"userspace access vector cache",
-										ALLOCSET_DEFAULT_MINSIZE,
-										ALLOCSET_DEFAULT_INITSIZE,
-										ALLOCSET_DEFAULT_MAXSIZE);
+										ALLOCSET_DEFAULT_SIZES);
 	memset(avc_slots, 0, sizeof(avc_slots));
 	avc_num_caches = 0;
 	avc_lru_hint = 0;
