@@ -3,7 +3,7 @@
  * async.c
  *	  Asynchronous notification: NOTIFY, LISTEN, UNLISTEN
  *
- * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -139,7 +139,6 @@
 #include "utils/ps_status.h"
 #include "utils/snapmgr.h"
 #include "utils/timestamp.h"
-#include "utils/tqual.h"
 
 
 /*
@@ -235,7 +234,7 @@ typedef struct QueueBackendStatus
  * When holding AsyncQueueLock in EXCLUSIVE mode, backends can inspect the
  * entries of other backends and also change the head pointer. When holding
  * both AsyncQueueLock and NotifyQueueTailLock in EXCLUSIVE mode, backends can
- * change the tail pointers.
+ * change the tail pointer.
  *
  * AsyncCtlLock is used as the control lock for the pg_notify SLRU buffers.
  * In order to avoid deadlocks, whenever we need multiple locks, we first get
@@ -250,8 +249,6 @@ typedef struct AsyncQueueControl
 	QueuePosition head;			/* head points to the next free location */
 	QueuePosition tail;			/* the global tail is equivalent to the pos of
 								 * the "slowest" backend */
-	int			stopPage;		/* oldest unrecycled page; must be <=
-								 * tail.page */
 	TimestampTz lastQueueFillWarn;	/* time of last queue-full msg */
 	QueueBackendStatus backend[FLEXIBLE_ARRAY_MEMBER];
 	/* backend[0] is not used; used entries are from [1] to [MaxBackends] */
@@ -261,7 +258,6 @@ static AsyncQueueControl *asyncQueueControl;
 
 #define QUEUE_HEAD					(asyncQueueControl->head)
 #define QUEUE_TAIL					(asyncQueueControl->tail)
-#define QUEUE_STOP_PAGE				(asyncQueueControl->stopPage)
 #define QUEUE_BACKEND_PID(i)		(asyncQueueControl->backend[i].pid)
 #define QUEUE_BACKEND_DBOID(i)		(asyncQueueControl->backend[i].dboid)
 #define QUEUE_BACKEND_POS(i)		(asyncQueueControl->backend[i].pos)
@@ -394,9 +390,9 @@ static void asyncQueueFillWarning(void);
 static bool SignalBackends(void);
 static void asyncQueueReadAllNotifications(void);
 static bool asyncQueueProcessPageEntries(volatile QueuePosition *current,
-							 QueuePosition stop,
-							 char *page_buffer,
-							 Snapshot snapshot);
+										 QueuePosition stop,
+										 char *page_buffer,
+										 Snapshot snapshot);
 static void asyncQueueAdvanceTail(void);
 static void ProcessIncomingNotify(void);
 static bool AsyncExistsPendingNotify(const char *channel, const char *payload);
@@ -404,9 +400,6 @@ static void ClearPendingActionsAndNotifies(void);
 
 /*
  * We will work on the page range of 0..QUEUE_MAX_PAGE.
- *
- * Since asyncQueueIsFull() blocks creation of a page that could precede any
- * extant page, we need not assess entries within a page.
  */
 static bool
 asyncQueuePagePrecedes(int p, int q)
@@ -474,7 +467,6 @@ AsyncShmemInit(void)
 
 		SET_QUEUE_POS(QUEUE_HEAD, 0, 0);
 		SET_QUEUE_POS(QUEUE_TAIL, 0, 0);
-		QUEUE_STOP_PAGE = 0;
 		asyncQueueControl->lastQueueFillWarn = 0;
 		/* zero'th entry won't be used, but let's initialize it anyway */
 		for (i = 0; i <= MaxBackends; i++)
@@ -1237,8 +1229,8 @@ asyncQueueIsFull(void)
 	 * logically precedes the current global tail pointer, ie, the head
 	 * pointer would wrap around compared to the tail.  We cannot create such
 	 * a head page for fear of confusing slru.c.  For safety we round the tail
-	 * pointer back to a segment boundary (truncation logic in
-	 * asyncQueueAdvanceTail does not do this, so doing it here is optional).
+	 * pointer back to a segment boundary (compare the truncation logic in
+	 * asyncQueueAdvanceTail).
 	 *
 	 * Note that this test is *not* dependent on how much space there is on
 	 * the current head page.  This is necessary because asyncQueueAddEntries
@@ -1247,7 +1239,7 @@ asyncQueueIsFull(void)
 	nexthead = QUEUE_POS_PAGE(QUEUE_HEAD) + 1;
 	if (nexthead > QUEUE_MAX_PAGE)
 		nexthead = 0;			/* wrap around */
-	boundary = QUEUE_STOP_PAGE;
+	boundary = QUEUE_POS_PAGE(QUEUE_TAIL);
 	boundary -= boundary % SLRU_PAGES_PER_SEGMENT;
 	return asyncQueuePagePrecedes(nexthead, boundary);
 }
@@ -1438,11 +1430,6 @@ pg_notification_queue_usage(PG_FUNCTION_ARGS)
  * Return the fraction of the queue that is currently occupied.
  *
  * The caller must hold AsyncQueueLock in (at least) shared mode.
- *
- * Note: we measure the distance to the logical tail page, not the physical
- * tail page.  In some sense that's wrong, but the relative position of the
- * physical tail is affected by details such as SLRU segment boundaries,
- * so that a result based on that is unpleasantly unstable.
  */
 static double
 asyncQueueUsage(void)
@@ -1973,7 +1960,7 @@ asyncQueueProcessPageEntries(volatile QueuePosition *current,
 				 * Note that we must test XidInMVCCSnapshot before we test
 				 * TransactionIdDidCommit, else we might return a message from
 				 * a transaction that is not yet visible to snapshots; compare
-				 * the comments at the head of tqual.c.
+				 * the comments at the head of heapam_visibility.c.
 				 *
 				 * Also, while our own xact won't be listed in the snapshot,
 				 * we need not check for TransactionIdIsCurrentTransactionId
@@ -2031,23 +2018,7 @@ asyncQueueAdvanceTail(void)
 	/* Restrict task to one backend per cluster; see SimpleLruTruncate(). */
 	LWLockAcquire(NotifyQueueTailLock, LW_EXCLUSIVE);
 
-	/*
-	 * Compute the new tail.  Pre-v13, it's essential that QUEUE_TAIL be exact
-	 * (ie, exactly match at least one backend's queue position), so it must
-	 * be updated atomically with the actual computation.  Since v13, we could
-	 * get away with not doing it like that, but it seems prudent to keep it
-	 * so.
-	 *
-	 * Also, because incoming backends will scan forward from QUEUE_TAIL, that
-	 * must be advanced before we can truncate any data.  Thus, QUEUE_TAIL is
-	 * the logical tail, while QUEUE_STOP_PAGE is the physical tail, or oldest
-	 * un-truncated page.  When QUEUE_STOP_PAGE != QUEUE_POS_PAGE(QUEUE_TAIL),
-	 * there are pages we can truncate but haven't yet finished doing so.
-	 *
-	 * For concurrency's sake, we don't want to hold AsyncQueueLock while
-	 * performing SimpleLruTruncate.  This is OK because no backend will try
-	 * to access the pages we are in the midst of truncating.
-	 */
+	/* Compute the new tail. */
 	LWLockAcquire(AsyncQueueLock, LW_EXCLUSIVE);
 	min = QUEUE_HEAD;
 	for (i = 1; i <= MaxBackends; i++)
@@ -2055,8 +2026,7 @@ asyncQueueAdvanceTail(void)
 		if (QUEUE_BACKEND_PID(i) != InvalidPid)
 			min = QUEUE_POS_MIN(min, QUEUE_BACKEND_POS(i));
 	}
-	QUEUE_TAIL = min;
-	oldtailpage = QUEUE_STOP_PAGE;
+	oldtailpage = QUEUE_POS_PAGE(QUEUE_TAIL);
 	LWLockRelease(AsyncQueueLock);
 
 	/*
@@ -2075,17 +2045,17 @@ asyncQueueAdvanceTail(void)
 		 * the lock again.
 		 */
 		SimpleLruTruncate(AsyncCtl, newtailpage);
-
-		/*
-		 * Update QUEUE_STOP_PAGE.  This changes asyncQueueIsFull()'s verdict
-		 * for the segment immediately prior to the old tail, allowing fresh
-		 * data into that segment.
-		 */
-		LWLockAcquire(AsyncQueueLock, LW_EXCLUSIVE);
-		QUEUE_STOP_PAGE = newtailpage;
-		LWLockRelease(AsyncQueueLock);
 	}
 
+	/*
+	 * Advertise the new tail.  This changes asyncQueueIsFull()'s verdict for
+	 * the segment immediately prior to the new tail, allowing fresh data into
+	 * that segment.
+	 */
+	LWLockAcquire(AsyncQueueLock, LW_EXCLUSIVE);
+	QUEUE_TAIL = min;
+	LWLockRelease(AsyncQueueLock);
+
 	LWLockRelease(NotifyQueueTailLock);
 }
 
