@@ -3,7 +3,7 @@
  * llvmjit.c
  *	  Core part of the LLVM JIT provider.
  *
- * Copyright (c) 2016-2018, PostgreSQL Global Development Group
+ * Copyright (c) 2016-2019, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/jit/llvm/llvmjit.c
@@ -29,13 +29,7 @@
 #include <llvm-c/BitWriter.h>
 #include <llvm-c/Core.h>
 #include <llvm-c/ExecutionEngine.h>
-#if LLVM_VERSION_MAJOR > 11
-#include <llvm-c/Orc.h>
-#include <llvm-c/OrcEE.h>
-#include <llvm-c/LLJIT.h>
-#else
 #include <llvm-c/OrcBindings.h>
-#endif
 #include <llvm-c/Support.h>
 #include <llvm-c/Target.h>
 #include <llvm-c/Transforms/IPO.h>
@@ -49,13 +43,8 @@
 /* Handle of a module emitted via ORC JIT */
 typedef struct LLVMJitHandle
 {
-#if LLVM_VERSION_MAJOR > 11
-	LLVMOrcLLJITRef lljit;
-	LLVMOrcResourceTrackerRef resource_tracker;
-#else
 	LLVMOrcJITStackRef stack;
 	LLVMOrcModuleHandle orc_handle;
-#endif
 } LLVMJitHandle;
 
 
@@ -64,6 +53,7 @@ LLVMTypeRef TypeSizeT;
 LLVMTypeRef TypeParamBool;
 LLVMTypeRef TypeStorageBool;
 LLVMTypeRef TypePGFunction;
+LLVMTypeRef StructNullableDatum;
 LLVMTypeRef StructHeapTupleFieldsField3;
 LLVMTypeRef StructHeapTupleFields;
 LLVMTypeRef StructHeapTupleHeaderData;
@@ -74,8 +64,10 @@ LLVMTypeRef StructItemPointerData;
 LLVMTypeRef StructBlockId;
 LLVMTypeRef StructFormPgAttribute;
 LLVMTypeRef StructTupleConstr;
-LLVMTypeRef StructtupleDesc;
+LLVMTypeRef StructTupleDescData;
 LLVMTypeRef StructTupleTableSlot;
+LLVMTypeRef StructHeapTupleTableSlot;
+LLVMTypeRef StructMinimalTupleTableSlot;
 LLVMTypeRef StructMemoryContextData;
 LLVMTypeRef StructPGFinfoRecord;
 LLVMTypeRef StructFmgrInfo;
@@ -90,11 +82,11 @@ LLVMTypeRef StructAggStatePerTransData;
 LLVMValueRef AttributeTemplate;
 LLVMValueRef FuncStrlen;
 LLVMValueRef FuncVarsizeAny;
-LLVMValueRef FuncSlotGetsomeattrs;
+LLVMValueRef FuncSlotGetsomeattrsInt;
 LLVMValueRef FuncSlotGetmissingattrs;
-LLVMValueRef FuncHeapGetsysattr;
 LLVMValueRef FuncMakeExpandedObjectReadOnlyInternal;
-LLVMValueRef FuncExecEvalArrayRefSubscript;
+LLVMValueRef FuncExecEvalSubscriptingRef;
+LLVMValueRef FuncExecEvalSysVar;
 LLVMValueRef FuncExecAggTransReparent;
 LLVMValueRef FuncExecAggInitGroup;
 
@@ -105,15 +97,12 @@ static const char *llvm_triple = NULL;
 static const char *llvm_layout = NULL;
 
 
+static LLVMTargetMachineRef llvm_opt0_targetmachine;
+static LLVMTargetMachineRef llvm_opt3_targetmachine;
+
 static LLVMTargetRef llvm_targetref;
-#if LLVM_VERSION_MAJOR > 11
-static LLVMOrcThreadSafeContextRef llvm_ts_context;
-static LLVMOrcLLJITRef llvm_opt0_orc;
-static LLVMOrcLLJITRef llvm_opt3_orc;
-#else							/* LLVM_VERSION_MAJOR > 11 */
 static LLVMOrcJITStackRef llvm_opt0_orc;
 static LLVMOrcJITStackRef llvm_opt3_orc;
-#endif							/* LLVM_VERSION_MAJOR > 11 */
 
 
 static void llvm_release_context(JitContext *context);
@@ -125,10 +114,6 @@ static void llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module);
 static void llvm_create_types(void);
 static uint64_t llvm_resolve_symbol(const char *name, void *ctx);
 
-#if LLVM_VERSION_MAJOR > 11
-static LLVMOrcLLJITRef llvm_create_jit_instance(LLVMTargetMachineRef tm);
-static char *llvm_error_message(LLVMErrorRef error);
-#endif							/* LLVM_VERSION_MAJOR > 11 */
 
 PG_MODULE_MAGIC;
 
@@ -181,54 +166,31 @@ llvm_release_context(JitContext *context)
 {
 	LLVMJitContext *llvm_context = (LLVMJitContext *) context;
 
+	llvm_enter_fatal_on_oom();
+
 	/*
 	 * When this backend is exiting, don't clean up LLVM. As an error might
 	 * have occurred from within LLVM, we do not want to risk reentering. All
 	 * resource cleanup is going to happen through process exit.
 	 */
-	if (proc_exit_inprogress)
-		return;
-
-	llvm_enter_fatal_on_oom();
-
-	if (llvm_context->module)
+	if (!proc_exit_inprogress)
 	{
-		LLVMDisposeModule(llvm_context->module);
-		llvm_context->module = NULL;
-	}
-
-	while (llvm_context->handles != NIL)
-	{
-		LLVMJitHandle *jit_handle;
-
-		jit_handle = (LLVMJitHandle *) linitial(llvm_context->handles);
-		llvm_context->handles = list_delete_first(llvm_context->handles);
-
-#if LLVM_VERSION_MAJOR > 11
+		if (llvm_context->module)
 		{
-			LLVMOrcExecutionSessionRef ee;
-			LLVMOrcSymbolStringPoolRef sp;
-
-			LLVMOrcResourceTrackerRemove(jit_handle->resource_tracker);
-			LLVMOrcReleaseResourceTracker(jit_handle->resource_tracker);
-
-			/*
-			 * Without triggering cleanup of the string pool, we'd leak
-			 * memory. It'd be sufficient to do this far less often, but in
-			 * experiments the required time was small enough to just always
-			 * do it.
-			 */
-			ee = LLVMOrcLLJITGetExecutionSession(jit_handle->lljit);
-			sp = LLVMOrcExecutionSessionGetSymbolStringPool(ee);
-			LLVMOrcSymbolStringPoolClearDeadEntries(sp);
+			LLVMDisposeModule(llvm_context->module);
+			llvm_context->module = NULL;
 		}
-#else							/* LLVM_VERSION_MAJOR > 11 */
+
+		while (llvm_context->handles != NIL)
 		{
+			LLVMJitHandle *jit_handle;
+
+			jit_handle = (LLVMJitHandle *) linitial(llvm_context->handles);
+			llvm_context->handles = list_delete_first(llvm_context->handles);
+
 			LLVMOrcRemoveModule(jit_handle->stack, jit_handle->orc_handle);
+			pfree(jit_handle);
 		}
-#endif							/* LLVM_VERSION_MAJOR > 11 */
-
-		pfree(jit_handle);
 	}
 }
 
@@ -284,8 +246,8 @@ llvm_expand_funcname(struct LLVMJitContext *context, const char *basename)
 void *
 llvm_get_function(LLVMJitContext *context, const char *funcname)
 {
-#if LLVM_VERSION_MAJOR > 11 || \
-	defined(HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN) && HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN
+	LLVMOrcTargetAddress addr = 0;
+#if defined(HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN) && HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN
 	ListCell   *lc;
 #endif
 
@@ -305,41 +267,10 @@ llvm_get_function(LLVMJitContext *context, const char *funcname)
 	 * to mangle here.
 	 */
 
-#if LLVM_VERSION_MAJOR > 11
+#if defined(HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN) && HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN
 	foreach(lc, context->handles)
 	{
 		LLVMJitHandle *handle = (LLVMJitHandle *) lfirst(lc);
-		instr_time	starttime;
-		instr_time	endtime;
-		LLVMErrorRef error;
-		LLVMOrcJITTargetAddress addr;
-
-		INSTR_TIME_SET_CURRENT(starttime);
-
-		addr = 0;
-		error = LLVMOrcLLJITLookup(handle->lljit, &addr, funcname);
-		if (error)
-			elog(ERROR, "failed to look up symbol \"%s\": %s",
-				 funcname, llvm_error_message(error));
-
-		/*
-		 * LLJIT only actually emits code the first time a symbol is
-		 * referenced. Thus add lookup time to emission time. That's counting
-		 * a bit more than with older LLVM versions, but unlikely to ever
-		 * matter.
-		 */
-		INSTR_TIME_SET_CURRENT(endtime);
-		INSTR_TIME_ACCUM_DIFF(context->base.instr.emission_counter,
-							  endtime, starttime);
-
-		if (addr)
-			return (void *) (uintptr_t) addr;
-	}
-#elif defined(HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN) && HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN
-	foreach(lc, context->handles)
-	{
-		LLVMOrcTargetAddress addr;
-		LLVMJitHandle *handle = (LLVMJitHandle *) lfirst(lc);
 
 		addr = 0;
 		if (LLVMOrcGetSymbolAddressIn(handle->stack, &addr, handle->orc_handle, funcname))
@@ -347,29 +278,26 @@ llvm_get_function(LLVMJitContext *context, const char *funcname)
 		if (addr)
 			return (void *) (uintptr_t) addr;
 	}
-#elif LLVM_VERSION_MAJOR < 5
-	{
-		LLVMOrcTargetAddress addr;
 
-		if ((addr = LLVMOrcGetSymbolAddress(llvm_opt0_orc, funcname)))
-			return (void *) (uintptr_t) addr;
-		if ((addr = LLVMOrcGetSymbolAddress(llvm_opt3_orc, funcname)))
-			return (void *) (uintptr_t) addr;
-	}
 #else
-	{
-		LLVMOrcTargetAddress addr;
 
-		if (LLVMOrcGetSymbolAddress(llvm_opt0_orc, &addr, funcname))
-			elog(ERROR, "failed to look up symbol \"%s\"", funcname);
-		if (addr)
-			return (void *) (uintptr_t) addr;
-		if (LLVMOrcGetSymbolAddress(llvm_opt3_orc, &addr, funcname))
-			elog(ERROR, "failed to look up symbol \"%s\"", funcname);
-		if (addr)
-			return (void *) (uintptr_t) addr;
-	}
-#endif
+#if LLVM_VERSION_MAJOR < 5
+	if ((addr = LLVMOrcGetSymbolAddress(llvm_opt0_orc, funcname)))
+		return (void *) (uintptr_t) addr;
+	if ((addr = LLVMOrcGetSymbolAddress(llvm_opt3_orc, funcname)))
+		return (void *) (uintptr_t) addr;
+#else
+	if (LLVMOrcGetSymbolAddress(llvm_opt0_orc, &addr, funcname))
+		elog(ERROR, "failed to look up symbol \"%s\"", funcname);
+	if (addr)
+		return (void *) (uintptr_t) addr;
+	if (LLVMOrcGetSymbolAddress(llvm_opt3_orc, &addr, funcname))
+		elog(ERROR, "failed to look up symbol \"%s\"", funcname);
+	if (addr)
+		return (void *) (uintptr_t) addr;
+#endif							/* LLVM_VERSION_MAJOR */
+
+#endif							/* HAVE_DECL_LLVMORCGETSYMBOLADDRESSIN */
 
 	elog(ERROR, "failed to JIT: %s", funcname);
 
@@ -410,7 +338,6 @@ llvm_copy_attributes_at_index(LLVMValueRef v_from, LLVMValueRef v_to, uint32 ind
 {
 	int			num_attributes;
 	LLVMAttributeRef *attrs;
-	int			attno;
 
 	num_attributes = LLVMGetAttributeCountAtIndexPG(v_from, index);
 
@@ -424,7 +351,7 @@ llvm_copy_attributes_at_index(LLVMValueRef v_from, LLVMValueRef v_to, uint32 ind
 	attrs = palloc(sizeof(LLVMAttributeRef) * num_attributes);
 	LLVMGetAttributesAtIndex(v_from, index, attrs);
 
-	for (attno = 0; attno < num_attributes; attno++)
+	for (int attno = 0; attno < num_attributes; attno++)
 		LLVMAddAttributeAtIndex(v_to, index, attrs[attno]);
 
 	pfree(attrs);
@@ -438,7 +365,6 @@ void
 llvm_copy_attributes(LLVMValueRef v_from, LLVMValueRef v_to)
 {
 	uint32		param_count;
-	int			paramidx;
 
 	/* copy function attributes */
 	llvm_copy_attributes_at_index(v_from, v_to, LLVMAttributeFunctionIndex);
@@ -449,7 +375,7 @@ llvm_copy_attributes(LLVMValueRef v_from, LLVMValueRef v_to)
 	/* and each function parameter's attribute */
 	param_count = LLVMCountParams(v_from);
 
-	for (paramidx = 1; paramidx <= param_count; paramidx++)
+	for (int paramidx = 1; paramidx <= param_count; paramidx++)
 		llvm_copy_attributes_at_index(v_from, v_to, paramidx);
 }
 
@@ -500,8 +426,6 @@ llvm_function_reference(LLVMJitContext *context,
 		v_fn = LLVMAddGlobal(mod, TypePGFunction, funcname);
 		LLVMSetInitializer(v_fn, v_fn_addr);
 		LLVMSetGlobalConstant(v_fn, true);
-		LLVMSetLinkage(v_fn, LLVMPrivateLinkage);
-		LLVMSetUnnamedAddr(v_fn, true);
 
 		return LLVMBuildLoad(builder, v_fn, "");
 	}
@@ -544,7 +468,7 @@ llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module)
 
 	if (context->base.flags & PGJIT_OPT3)
 	{
-		/* TODO: Unscientifically determined threshhold */
+		/* TODO: Unscientifically determined threshold */
 		LLVMPassManagerBuilderUseInlinerWithThreshold(llvm_pmb, 512);
 	}
 	else
@@ -593,15 +517,11 @@ llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module)
 static void
 llvm_compile_module(LLVMJitContext *context)
 {
-	LLVMJitHandle *handle;
+	LLVMOrcModuleHandle orc_handle;
 	MemoryContext oldcontext;
+	static LLVMOrcJITStackRef compile_orc;
 	instr_time	starttime;
 	instr_time	endtime;
-#if LLVM_VERSION_MAJOR > 11
-	LLVMOrcLLJITRef compile_orc;
-#else
-	LLVMOrcJITStackRef compile_orc;
-#endif
 
 	if (context->base.flags & PGJIT_OPT3)
 		compile_orc = llvm_opt3_orc;
@@ -648,9 +568,6 @@ llvm_compile_module(LLVMJitContext *context)
 		pfree(filename);
 	}
 
-	handle = (LLVMJitHandle *)
-		MemoryContextAlloc(TopMemoryContext, sizeof(LLVMJitHandle));
-
 	/*
 	 * Emit the code. Note that this can, depending on the optimization
 	 * settings, take noticeable resources as code emission executes low-level
@@ -658,42 +575,13 @@ llvm_compile_module(LLVMJitContext *context)
 	 * faster instruction selection mechanism is used.
 	 */
 	INSTR_TIME_SET_CURRENT(starttime);
-#if LLVM_VERSION_MAJOR > 11
+#if LLVM_VERSION_MAJOR > 6
 	{
-		LLVMOrcThreadSafeModuleRef ts_module;
-		LLVMErrorRef error;
-		LLVMOrcJITDylibRef jd = LLVMOrcLLJITGetMainJITDylib(compile_orc);
-
-		ts_module = LLVMOrcCreateNewThreadSafeModule(context->module, llvm_ts_context);
-
-		handle->lljit = compile_orc;
-		handle->resource_tracker = LLVMOrcJITDylibCreateResourceTracker(jd);
-
-		/*
-		 * NB: This doesn't actually emit code. That happens lazily the first
-		 * time a symbol defined in the module is requested. Due to that
-		 * llvm_get_function() also accounts for emission time.
-		 */
-
-		context->module = NULL; /* will be owned by LLJIT */
-		error = LLVMOrcLLJITAddLLVMIRModuleWithRT(compile_orc,
-												  handle->resource_tracker,
-												  ts_module);
-
-		if (error)
-			elog(ERROR, "failed to JIT module: %s",
-				 llvm_error_message(error));
-
-		handle->lljit = compile_orc;
-
-		/* LLVMOrcLLJITAddLLVMIRModuleWithRT takes ownership of the module */
-	}
-#elif LLVM_VERSION_MAJOR > 6
-	{
-		handle->stack = compile_orc;
-		if (LLVMOrcAddEagerlyCompiledIR(compile_orc, &handle->orc_handle, context->module,
+		if (LLVMOrcAddEagerlyCompiledIR(compile_orc, &orc_handle, context->module,
 										llvm_resolve_symbol, NULL))
+		{
 			elog(ERROR, "failed to JIT module");
+		}
 
 		/* LLVMOrcAddEagerlyCompiledIR takes ownership of the module */
 	}
@@ -702,23 +590,20 @@ llvm_compile_module(LLVMJitContext *context)
 		LLVMSharedModuleRef smod;
 
 		smod = LLVMOrcMakeSharedModule(context->module);
-		handle->stack = compile_orc;
-		if (LLVMOrcAddEagerlyCompiledIR(compile_orc, &handle->orc_handle, smod,
+		if (LLVMOrcAddEagerlyCompiledIR(compile_orc, &orc_handle, smod,
 										llvm_resolve_symbol, NULL))
+		{
 			elog(ERROR, "failed to JIT module");
-
+		}
 		LLVMOrcDisposeSharedModuleRef(smod);
 	}
 #else							/* LLVM 4.0 and 3.9 */
 	{
-		handle->stack = compile_orc;
-		handle->orc_handle = LLVMOrcAddEagerlyCompiledIR(compile_orc, context->module,
-														 llvm_resolve_symbol, NULL);
-
+		orc_handle = LLVMOrcAddEagerlyCompiledIR(compile_orc, context->module,
+												 llvm_resolve_symbol, NULL);
 		LLVMDisposeModule(context->module);
 	}
 #endif
-
 	INSTR_TIME_SET_CURRENT(endtime);
 	INSTR_TIME_ACCUM_DIFF(context->base.instr.emission_counter,
 						  endtime, starttime);
@@ -728,7 +613,15 @@ llvm_compile_module(LLVMJitContext *context)
 
 	/* remember emitted code for cleanup and lookups */
 	oldcontext = MemoryContextSwitchTo(TopMemoryContext);
-	context->handles = lappend(context->handles, handle);
+	{
+		LLVMJitHandle *handle;
+
+		handle = (LLVMJitHandle *) palloc(sizeof(LLVMJitHandle));
+		handle->stack = compile_orc;
+		handle->orc_handle = orc_handle;
+
+		context->handles = lappend(context->handles, handle);
+	}
 	MemoryContextSwitchTo(oldcontext);
 
 	ereport(DEBUG1,
@@ -750,8 +643,6 @@ llvm_session_initialize(void)
 	char	   *error = NULL;
 	char	   *cpu = NULL;
 	char	   *features = NULL;
-	LLVMTargetMachineRef opt0_tm;
-	LLVMTargetMachineRef opt3_tm;
 
 	if (llvm_session_initialized)
 		return;
@@ -762,16 +653,6 @@ llvm_session_initialize(void)
 	LLVMInitializeNativeAsmPrinter();
 	LLVMInitializeNativeAsmParser();
 
-	/*
-	 * When targeting an LLVM version with opaque pointers enabled by
-	 * default, turn them off for the context we build our code in.  We don't
-	 * need to do so for other contexts (e.g. llvm_ts_context).  Once the IR is
-	 * generated, it carries the necessary information.
-	 */
-#if LLVM_VERSION_MAJOR > 14
-	LLVMContextSetOpaquePointers(LLVMGetGlobalContext(), false);
-#endif
-
 	/*
 	 * Synchronize types early, as that also includes inferring the target
 	 * triple.
@@ -780,7 +661,7 @@ llvm_session_initialize(void)
 
 	if (LLVMGetTargetFromTriple(llvm_triple, &llvm_targetref, &error) != 0)
 	{
-		elog(FATAL, "failed to query triple %s", error);
+		elog(FATAL, "failed to query triple %s\n", error);
 	}
 
 	/*
@@ -794,12 +675,12 @@ llvm_session_initialize(void)
 	elog(DEBUG2, "LLVMJIT detected CPU \"%s\", with features \"%s\"",
 		 cpu, features);
 
-	opt0_tm =
+	llvm_opt0_targetmachine =
 		LLVMCreateTargetMachine(llvm_targetref, llvm_triple, cpu, features,
 								LLVMCodeGenLevelNone,
 								LLVMRelocDefault,
 								LLVMCodeModelJITDefault);
-	opt3_tm =
+	llvm_opt3_targetmachine =
 		LLVMCreateTargetMachine(llvm_targetref, llvm_triple, cpu, features,
 								LLVMCodeGenLevelAggressive,
 								LLVMRelocDefault,
@@ -813,41 +694,27 @@ llvm_session_initialize(void)
 	/* force symbols in main binary to be loaded */
 	LLVMLoadLibraryPermanently(NULL);
 
-#if LLVM_VERSION_MAJOR > 11
-	{
-		llvm_ts_context = LLVMOrcCreateNewThreadSafeContext();
-
-		llvm_opt0_orc = llvm_create_jit_instance(opt0_tm);
-		opt0_tm = 0;
-
-		llvm_opt3_orc = llvm_create_jit_instance(opt3_tm);
-		opt3_tm = 0;
-	}
-#else							/* LLVM_VERSION_MAJOR > 11 */
-	{
-		llvm_opt0_orc = LLVMOrcCreateInstance(opt0_tm);
-		llvm_opt3_orc = LLVMOrcCreateInstance(opt3_tm);
+	llvm_opt0_orc = LLVMOrcCreateInstance(llvm_opt0_targetmachine);
+	llvm_opt3_orc = LLVMOrcCreateInstance(llvm_opt3_targetmachine);
 
 #if defined(HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER) && HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER
-		if (jit_debugging_support)
-		{
-			LLVMJITEventListenerRef l = LLVMCreateGDBRegistrationListener();
+	if (jit_debugging_support)
+	{
+		LLVMJITEventListenerRef l = LLVMCreateGDBRegistrationListener();
 
-			LLVMOrcRegisterJITEventListener(llvm_opt0_orc, l);
-			LLVMOrcRegisterJITEventListener(llvm_opt3_orc, l);
-		}
+		LLVMOrcRegisterJITEventListener(llvm_opt0_orc, l);
+		LLVMOrcRegisterJITEventListener(llvm_opt3_orc, l);
+	}
 #endif
 #if defined(HAVE_DECL_LLVMCREATEPERFJITEVENTLISTENER) && HAVE_DECL_LLVMCREATEPERFJITEVENTLISTENER
-		if (jit_profiling_support)
-		{
-			LLVMJITEventListenerRef l = LLVMCreatePerfJITEventListener();
+	if (jit_profiling_support)
+	{
+		LLVMJITEventListenerRef l = LLVMCreatePerfJITEventListener();
 
-			LLVMOrcRegisterJITEventListener(llvm_opt0_orc, l);
-			LLVMOrcRegisterJITEventListener(llvm_opt3_orc, l);
-		}
-#endif
+		LLVMOrcRegisterJITEventListener(llvm_opt0_orc, l);
+		LLVMOrcRegisterJITEventListener(llvm_opt3_orc, l);
 	}
-#endif							/* LLVM_VERSION_MAJOR > 11 */
+#endif
 
 	before_shmem_exit(llvm_shutdown, 0);
 
@@ -859,63 +726,27 @@ llvm_session_initialize(void)
 static void
 llvm_shutdown(int code, Datum arg)
 {
-	/*
-	 * If llvm_shutdown() is reached while in a fatal-on-oom section an error
-	 * has occurred in the middle of LLVM code. It is not safe to call back
-	 * into LLVM (which is why a FATAL error was thrown).
-	 *
-	 * We do need to shutdown LLVM in other shutdown cases, otherwise
-	 * e.g. profiling data won't be written out.
-	 */
-	if (llvm_in_fatal_on_oom())
-	{
-		Assert(proc_exit_inprogress);
-		return;
-	}
+	/* unregister profiling support, needs to be flushed to be useful */
 
-#if LLVM_VERSION_MAJOR > 11
+	if (llvm_opt3_orc)
 	{
-		if (llvm_opt3_orc)
-		{
-			LLVMOrcDisposeLLJIT(llvm_opt3_orc);
-			llvm_opt3_orc = NULL;
-		}
-		if (llvm_opt0_orc)
-		{
-			LLVMOrcDisposeLLJIT(llvm_opt0_orc);
-			llvm_opt0_orc = NULL;
-		}
-		if (llvm_ts_context)
-		{
-			LLVMOrcDisposeThreadSafeContext(llvm_ts_context);
-			llvm_ts_context = NULL;
-		}
-	}
-#else							/* LLVM_VERSION_MAJOR > 11 */
-	{
-		/* unregister profiling support, needs to be flushed to be useful */
-
-		if (llvm_opt3_orc)
-		{
 #if defined(HAVE_DECL_LLVMORCREGISTERPERF) && HAVE_DECL_LLVMORCREGISTERPERF
-			if (jit_profiling_support)
-				LLVMOrcUnregisterPerf(llvm_opt3_orc);
+		if (jit_profiling_support)
+			LLVMOrcUnregisterPerf(llvm_opt3_orc);
 #endif
-			LLVMOrcDisposeInstance(llvm_opt3_orc);
-			llvm_opt3_orc = NULL;
-		}
+		LLVMOrcDisposeInstance(llvm_opt3_orc);
+		llvm_opt3_orc = NULL;
+	}
 
-		if (llvm_opt0_orc)
-		{
+	if (llvm_opt0_orc)
+	{
 #if defined(HAVE_DECL_LLVMORCREGISTERPERF) && HAVE_DECL_LLVMORCREGISTERPERF
-			if (jit_profiling_support)
-				LLVMOrcUnregisterPerf(llvm_opt0_orc);
+		if (jit_profiling_support)
+			LLVMOrcUnregisterPerf(llvm_opt0_orc);
 #endif
-			LLVMOrcDisposeInstance(llvm_opt0_orc);
-			llvm_opt0_orc = NULL;
-		}
+		LLVMOrcDisposeInstance(llvm_opt0_orc);
+		llvm_opt0_orc = NULL;
 	}
-#endif							/* LLVM_VERSION_MAJOR > 11 */
 }
 
 /* helper for llvm_create_types, returning a global var's type */
@@ -1004,14 +835,17 @@ llvm_create_types(void)
 	TypeParamBool = load_return_type(mod, "FunctionReturningBool");
 	TypeStorageBool = load_type(mod, "TypeStorageBool");
 	TypePGFunction = load_type(mod, "TypePGFunction");
+	StructNullableDatum = load_type(mod, "StructNullableDatum");
 	StructExprContext = load_type(mod, "StructExprContext");
 	StructExprEvalStep = load_type(mod, "StructExprEvalStep");
 	StructExprState = load_type(mod, "StructExprState");
 	StructFunctionCallInfoData = load_type(mod, "StructFunctionCallInfoData");
 	StructMemoryContextData = load_type(mod, "StructMemoryContextData");
 	StructTupleTableSlot = load_type(mod, "StructTupleTableSlot");
+	StructHeapTupleTableSlot = load_type(mod, "StructHeapTupleTableSlot");
+	StructMinimalTupleTableSlot = load_type(mod, "StructMinimalTupleTableSlot");
 	StructHeapTupleData = load_type(mod, "StructHeapTupleData");
-	StructtupleDesc = load_type(mod, "StructtupleDesc");
+	StructTupleDescData = load_type(mod, "StructTupleDescData");
 	StructAggState = load_type(mod, "StructAggState");
 	StructAggStatePerGroupData = load_type(mod, "StructAggStatePerGroupData");
 	StructAggStatePerTransData = load_type(mod, "StructAggStatePerTransData");
@@ -1019,11 +853,11 @@ llvm_create_types(void)
 	AttributeTemplate = LLVMGetNamedFunction(mod, "AttributeTemplate");
 	FuncStrlen = LLVMGetNamedFunction(mod, "strlen");
 	FuncVarsizeAny = LLVMGetNamedFunction(mod, "varsize_any");
-	FuncSlotGetsomeattrs = LLVMGetNamedFunction(mod, "slot_getsomeattrs");
+	FuncSlotGetsomeattrsInt = LLVMGetNamedFunction(mod, "slot_getsomeattrs_int");
 	FuncSlotGetmissingattrs = LLVMGetNamedFunction(mod, "slot_getmissingattrs");
-	FuncHeapGetsysattr = LLVMGetNamedFunction(mod, "heap_getsysattr");
 	FuncMakeExpandedObjectReadOnlyInternal = LLVMGetNamedFunction(mod, "MakeExpandedObjectReadOnlyInternal");
-	FuncExecEvalArrayRefSubscript = LLVMGetNamedFunction(mod, "ExecEvalArrayRefSubscript");
+	FuncExecEvalSubscriptingRef = LLVMGetNamedFunction(mod, "ExecEvalSubscriptingRef");
+	FuncExecEvalSysVar = LLVMGetNamedFunction(mod, "ExecEvalSysVar");
 	FuncExecAggTransReparent = LLVMGetNamedFunction(mod, "ExecAggTransReparent");
 	FuncExecAggInitGroup = LLVMGetNamedFunction(mod, "ExecAggInitGroup");
 
@@ -1052,7 +886,7 @@ llvm_split_symbol_name(const char *name, char **modname, char **funcname)
 	{
 		/*
 		 * Symbol names cannot contain a ., therefore we can split based on
-		 * first and last occurance of one.
+		 * first and last occurrence of one.
 		 */
 		*funcname = rindex(name, '.');
 		(*funcname)++;			/* jump over . */
@@ -1111,156 +945,3 @@ llvm_resolve_symbol(const char *symname, void *ctx)
 
 	return (uint64_t) addr;
 }
-
-#if LLVM_VERSION_MAJOR > 11
-
-static LLVMErrorRef
-llvm_resolve_symbols(LLVMOrcDefinitionGeneratorRef GeneratorObj, void *Ctx,
-					 LLVMOrcLookupStateRef *LookupState, LLVMOrcLookupKind Kind,
-					 LLVMOrcJITDylibRef JD, LLVMOrcJITDylibLookupFlags JDLookupFlags,
-					 LLVMOrcCLookupSet LookupSet, size_t LookupSetSize)
-{
-#if LLVM_VERSION_MAJOR > 14
-	LLVMOrcCSymbolMapPairs symbols = palloc0(sizeof(LLVMOrcCSymbolMapPair) * LookupSetSize);
-#else
-	LLVMOrcCSymbolMapPairs symbols = palloc0(sizeof(LLVMJITCSymbolMapPair) * LookupSetSize);
-#endif
-	LLVMErrorRef error;
-	LLVMOrcMaterializationUnitRef mu;
-
-	for (int i = 0; i < LookupSetSize; i++)
-	{
-		const char *name = LLVMOrcSymbolStringPoolEntryStr(LookupSet[i].Name);
-
-#if LLVM_VERSION_MAJOR > 12
-		LLVMOrcRetainSymbolStringPoolEntry(LookupSet[i].Name);
-#endif
-		symbols[i].Name = LookupSet[i].Name;
-		symbols[i].Sym.Address = llvm_resolve_symbol(name, NULL);
-		symbols[i].Sym.Flags.GenericFlags = LLVMJITSymbolGenericFlagsExported;
-	}
-
-	mu = LLVMOrcAbsoluteSymbols(symbols, LookupSetSize);
-	error = LLVMOrcJITDylibDefine(JD, mu);
-	if (error != LLVMErrorSuccess)
-		LLVMOrcDisposeMaterializationUnit(mu);
-
-	pfree(symbols);
-
-	return error;
-}
-
-/*
- * We cannot throw errors through LLVM (without causing a FATAL at least), so
- * just use WARNING here. That's OK anyway, as the error is also reported at
- * the top level action (with less detail) and there might be multiple
- * invocations of errors with details.
- *
- * This doesn't really happen during normal operation, but in cases like
- * symbol resolution breakage. So just using elog(WARNING) is fine.
- */
-static void
-llvm_log_jit_error(void *ctx, LLVMErrorRef error)
-{
-	elog(WARNING, "error during JITing: %s",
-		 llvm_error_message(error));
-}
-
-/*
- * Create our own object layer, so we can add event listeners.
- */
-static LLVMOrcObjectLayerRef
-llvm_create_object_layer(void *Ctx, LLVMOrcExecutionSessionRef ES, const char *Triple)
-{
-	LLVMOrcObjectLayerRef objlayer =
-	LLVMOrcCreateRTDyldObjectLinkingLayerWithSectionMemoryManager(ES);
-
-#if defined(HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER) && HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER
-	if (jit_debugging_support)
-	{
-		LLVMJITEventListenerRef l = LLVMCreateGDBRegistrationListener();
-
-		LLVMOrcRTDyldObjectLinkingLayerRegisterJITEventListener(objlayer, l);
-	}
-#endif
-
-#if defined(HAVE_DECL_LLVMCREATEPERFJITEVENTLISTENER) && HAVE_DECL_LLVMCREATEPERFJITEVENTLISTENER
-	if (jit_profiling_support)
-	{
-		LLVMJITEventListenerRef l = LLVMCreatePerfJITEventListener();
-
-		LLVMOrcRTDyldObjectLinkingLayerRegisterJITEventListener(objlayer, l);
-	}
-#endif
-
-	return objlayer;
-}
-
-/*
- * Create LLJIT instance, using the passed in target machine. Note that the
- * target machine afterwards is owned by the LLJIT instance.
- */
-static LLVMOrcLLJITRef
-llvm_create_jit_instance(LLVMTargetMachineRef tm)
-{
-	LLVMOrcLLJITRef lljit;
-	LLVMOrcJITTargetMachineBuilderRef tm_builder;
-	LLVMOrcLLJITBuilderRef lljit_builder;
-	LLVMErrorRef error;
-	LLVMOrcDefinitionGeneratorRef main_gen;
-	LLVMOrcDefinitionGeneratorRef ref_gen;
-
-	lljit_builder = LLVMOrcCreateLLJITBuilder();
-	tm_builder = LLVMOrcJITTargetMachineBuilderCreateFromTargetMachine(tm);
-	LLVMOrcLLJITBuilderSetJITTargetMachineBuilder(lljit_builder, tm_builder);
-
-	LLVMOrcLLJITBuilderSetObjectLinkingLayerCreator(lljit_builder,
-													llvm_create_object_layer,
-													NULL);
-
-	error = LLVMOrcCreateLLJIT(&lljit, lljit_builder);
-	if (error)
-		elog(ERROR, "failed to create lljit instance: %s",
-			 llvm_error_message(error));
-
-	LLVMOrcExecutionSessionSetErrorReporter(LLVMOrcLLJITGetExecutionSession(lljit),
-											llvm_log_jit_error, NULL);
-
-	/*
-	 * Symbol resolution support for symbols in the postgres binary /
-	 * libraries already loaded.
-	 */
-	error = LLVMOrcCreateDynamicLibrarySearchGeneratorForProcess(&main_gen,
-																 LLVMOrcLLJITGetGlobalPrefix(lljit),
-																 0, NULL);
-	if (error)
-		elog(ERROR, "failed to create generator: %s",
-			 llvm_error_message(error));
-	LLVMOrcJITDylibAddGenerator(LLVMOrcLLJITGetMainJITDylib(lljit), main_gen);
-
-	/*
-	 * Symbol resolution support for "special" functions, e.g. a call into an
-	 * SQL callable function.
-	 */
-#if LLVM_VERSION_MAJOR > 14
-	ref_gen = LLVMOrcCreateCustomCAPIDefinitionGenerator(llvm_resolve_symbols, NULL, NULL);
-#else
-	ref_gen = LLVMOrcCreateCustomCAPIDefinitionGenerator(llvm_resolve_symbols, NULL);
-#endif
-	LLVMOrcJITDylibAddGenerator(LLVMOrcLLJITGetMainJITDylib(lljit), ref_gen);
-
-	return lljit;
-}
-
-static char *
-llvm_error_message(LLVMErrorRef error)
-{
-	char	   *orig = LLVMGetErrorMessage(error);
-	char	   *msg = pstrdup(orig);
-
-	LLVMDisposeErrorMessage(orig);
-
-	return msg;
-}
-
-#endif							/* LLVM_VERSION_MAJOR > 11 */
