@@ -20,7 +20,7 @@
  * appropriate value for a free lock.  The meaning of the variable is up to
  * the caller, the lightweight lock code just assigns and compares it.
  *
- * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -522,6 +522,7 @@ RegisterLWLockTranches(void)
 	LWLockRegisterTranche(LWTRANCHE_TBM, "tbm");
 	LWLockRegisterTranche(LWTRANCHE_PARALLEL_APPEND, "parallel_append");
 	LWLockRegisterTranche(LWTRANCHE_PARALLEL_HASH_JOIN, "parallel_hash_join");
+	LWLockRegisterTranche(LWTRANCHE_SXACT, "serializable_xact");
 
 	/* Register named tranches. */
 	for (i = 0; i < NamedLWLockTrancheRequests; i++)
@@ -1221,10 +1222,14 @@ LWLockAcquire(LWLock *lock, LWLockMode mode)
 		/*
 		 * Wait until awakened.
 		 *
-		 * It is possible that we get awakened for a reason other than being
-		 * signaled by LWLockRelease.  If so, loop back and wait again.  Once
-		 * we've gotten the LWLock, re-increment the sema by the number of
-		 * additional signals received.
+		 * Since we share the process wait semaphore with the regular lock
+		 * manager and ProcWaitForSignal, and we may need to acquire an LWLock
+		 * while one of those is pending, it is possible that we get awakened
+		 * for a reason other than being signaled by LWLockRelease. If so,
+		 * loop back and wait again.  Once we've gotten the LWLock,
+		 * re-increment the sema by the number of additional signals received,
+		 * so that the lock manager or signal manager will see the received
+		 * signal when it next waits.
 		 */
 		LOG_LWDEBUG("LWLockAcquire", lock, "waiting");
 
@@ -1384,7 +1389,8 @@ LWLockAcquireOrWait(LWLock *lock, LWLockMode mode)
 		{
 			/*
 			 * Wait until awakened.  Like in LWLockAcquire, be prepared for
-			 * bogus wakeups.
+			 * bogus wakeups, because we share the semaphore with
+			 * ProcWaitForSignal.
 			 */
 			LOG_LWDEBUG("LWLockAcquireOrWait", lock, "waiting");
 
@@ -1593,10 +1599,14 @@ LWLockWaitForVar(LWLock *lock, uint64 *valptr, uint64 oldval, uint64 *newval)
 		/*
 		 * Wait until awakened.
 		 *
-		 * It is possible that we get awakened for a reason other than being
-		 * signaled by LWLockRelease.  If so, loop back and wait again.  Once
-		 * we've gotten the LWLock, re-increment the sema by the number of
-		 * additional signals received.
+		 * Since we share the process wait semaphore with the regular lock
+		 * manager and ProcWaitForSignal, and we may need to acquire an LWLock
+		 * while one of those is pending, it is possible that we get awakened
+		 * for a reason other than being signaled by LWLockRelease. If so,
+		 * loop back and wait again.  Once we've gotten the LWLock,
+		 * re-increment the sema by the number of additional signals received,
+		 * so that the lock manager or signal manager will see the received
+		 * signal when it next waits.
 		 */
 		LOG_LWDEBUG("LWLockWaitForVar", lock, "waiting");
 
@@ -1842,32 +1852,6 @@ LWLockHeldByMe(LWLock *l)
 	return false;
 }
 
-/*
- * LWLockHeldByMe - test whether my process holds any of an array of locks
- *
- * This is meant as debug support only.
- */
-bool
-LWLockAnyHeldByMe(LWLock *l, int nlocks, size_t stride)
-{
-	char	   *held_lock_addr;
-	char	   *begin;
-	char	   *end;
-	int			i;
-
-	begin = (char *) l;
-	end = begin + nlocks * stride;
-	for (i = 0; i < num_held_lwlocks; i++)
-	{
-		held_lock_addr = (char *) held_lwlocks[i].lock;
-		if (held_lock_addr >= begin &&
-			held_lock_addr < end &&
-			(held_lock_addr - begin) % stride == 0)
-			return true;
-	}
-	return false;
-}
-
 /*
  * LWLockHeldByMeInMode - test whether my process holds a lock in given mode
  *
