@@ -17,7 +17,7 @@
  *	sync.
  *
  *
- *	Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
+ *	Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
  *	Portions Copyright (c) 1994, Regents of the University of California
  *	Portions Copyright (c) 2000, Philip Warner
  *
@@ -25,7 +25,7 @@
  *	as this notice is not removed.
  *
  *	The author is not responsible for loss or damages that may
- *	result from it's use.
+ *	result from its use.
  *
  * IDENTIFICATION
  *		src/bin/pg_dump/pg_backup_directory.c
@@ -61,9 +61,6 @@ typedef struct
 	char	   *filename;		/* filename excluding the directory (basename) */
 } lclTocEntry;
 
-/* translator: this is a module name */
-static const char *modulename = gettext_noop("directory archiver");
-
 /* prototypes for private functions */
 static void _ArchiveEntry(ArchiveHandle *AH, TocEntry *te);
 static void _StartData(ArchiveHandle *AH, TocEntry *te);
@@ -87,6 +84,7 @@ static void _EndBlob(ArchiveHandle *AH, TocEntry *te, Oid oid);
 static void _EndBlobs(ArchiveHandle *AH, TocEntry *te);
 static void _LoadBlobs(ArchiveHandle *AH);
 
+static void _PrepParallelRestore(ArchiveHandle *AH);
 static void _Clone(ArchiveHandle *AH);
 static void _DeClone(ArchiveHandle *AH);
 
@@ -94,7 +92,7 @@ static int	_WorkerJobRestoreDirectory(ArchiveHandle *AH, TocEntry *te);
 static int	_WorkerJobDumpDirectory(ArchiveHandle *AH, TocEntry *te);
 
 static void setFilePath(ArchiveHandle *AH, char *buf,
-			const char *relativeFilename);
+						const char *relativeFilename);
 
 /*
  *	Init routine required by ALL formats. This is a global routine
@@ -132,6 +130,7 @@ InitArchiveFmt_Directory(ArchiveHandle *AH)
 	AH->EndBlobPtr = _EndBlob;
 	AH->EndBlobsPtr = _EndBlobs;
 
+	AH->PrepParallelRestorePtr = _PrepParallelRestore;
 	AH->ClonePtr = _Clone;
 	AH->DeClonePtr = _DeClone;
 
@@ -154,7 +153,7 @@ InitArchiveFmt_Directory(ArchiveHandle *AH)
 	 */
 
 	if (!AH->fSpec || strcmp(AH->fSpec, "") == 0)
-		exit_horribly(modulename, "no output directory specified\n");
+		fatal("no output directory specified");
 
 	ctx->directory = AH->fSpec;
 
@@ -183,18 +182,18 @@ InitArchiveFmt_Directory(ArchiveHandle *AH)
 				}
 
 				if (errno)
-					exit_horribly(modulename, "could not read directory \"%s\": %s\n",
-								  ctx->directory, strerror(errno));
+					fatal("could not read directory \"%s\": %m",
+						  ctx->directory);
 
 				if (closedir(dir))
-					exit_horribly(modulename, "could not close directory \"%s\": %s\n",
-								  ctx->directory, strerror(errno));
+					fatal("could not close directory \"%s\": %m",
+						  ctx->directory);
 			}
 		}
 
 		if (!is_empty && mkdir(ctx->directory, 0700) < 0)
-			exit_horribly(modulename, "could not create directory \"%s\": %s\n",
-						  ctx->directory, strerror(errno));
+			fatal("could not create directory \"%s\": %m",
+				  ctx->directory);
 	}
 	else
 	{							/* Read Mode */
@@ -205,9 +204,7 @@ InitArchiveFmt_Directory(ArchiveHandle *AH)
 
 		tocFH = cfopen_read(fname, PG_BINARY_R);
 		if (tocFH == NULL)
-			exit_horribly(modulename,
-						  "could not open input file \"%s\": %s\n",
-						  fname, strerror(errno));
+			fatal("could not open input file \"%s\": %m", fname);
 
 		ctx->dataFH = tocFH;
 
@@ -222,8 +219,7 @@ InitArchiveFmt_Directory(ArchiveHandle *AH)
 
 		/* Nothing else in the file, so close it again... */
 		if (cfclose(tocFH) != 0)
-			exit_horribly(modulename, "could not close TOC file: %s\n",
-						  strerror(errno));
+			fatal("could not close TOC file: %m");
 		ctx->dataFH = NULL;
 	}
 }
@@ -240,13 +236,13 @@ _ArchiveEntry(ArchiveHandle *AH, TocEntry *te)
 	char		fn[MAXPGPATH];
 
 	tctx = (lclTocEntry *) pg_malloc0(sizeof(lclTocEntry));
-	if (te->dataDumper)
+	if (strcmp(te->desc, "BLOBS") == 0)
+		tctx->filename = pg_strdup("blobs.toc");
+	else if (te->dataDumper)
 	{
 		snprintf(fn, MAXPGPATH, "%d.dat", te->dumpId);
 		tctx->filename = pg_strdup(fn);
 	}
-	else if (strcmp(te->desc, "BLOBS") == 0)
-		tctx->filename = pg_strdup("blobs.toc");
 	else
 		tctx->filename = NULL;
 
@@ -333,8 +329,7 @@ _StartData(ArchiveHandle *AH, TocEntry *te)
 
 	ctx->dataFH = cfopen_write(fname, PG_BINARY_W, AH->compression);
 	if (ctx->dataFH == NULL)
-		exit_horribly(modulename, "could not open output file \"%s\": %s\n",
-					  fname, strerror(errno));
+		fatal("could not open output file \"%s\": %m", fname);
 }
 
 /*
@@ -357,8 +352,8 @@ _WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 		/* if write didn't set errno, assume problem is no disk space */
 		if (errno == 0)
 			errno = ENOSPC;
-		exit_horribly(modulename, "could not write to output file: %s\n",
-					  get_cfp_error(ctx->dataFH));
+		fatal("could not write to output file: %s",
+			  get_cfp_error(ctx->dataFH));
 	}
 
 	return;
@@ -398,8 +393,7 @@ _PrintFileData(ArchiveHandle *AH, char *filename)
 	cfp = cfopen_read(filename, PG_BINARY_R);
 
 	if (!cfp)
-		exit_horribly(modulename, "could not open input file \"%s\": %s\n",
-					  filename, strerror(errno));
+		fatal("could not open input file \"%s\": %m", filename);
 
 	buf = pg_malloc(ZLIB_OUT_SIZE);
 	buflen = ZLIB_OUT_SIZE;
@@ -411,8 +405,7 @@ _PrintFileData(ArchiveHandle *AH, char *filename)
 
 	free(buf);
 	if (cfclose(cfp) !=0)
-		exit_horribly(modulename, "could not close data file \"%s\": %s\n",
-					  filename, strerror(errno));
+		fatal("could not close data file \"%s\": %m", filename);
 }
 
 /*
@@ -452,8 +445,8 @@ _LoadBlobs(ArchiveHandle *AH)
 	ctx->blobsTocFH = cfopen_read(fname, PG_BINARY_R);
 
 	if (ctx->blobsTocFH == NULL)
-		exit_horribly(modulename, "could not open large object TOC file \"%s\" for input: %s\n",
-					  fname, strerror(errno));
+		fatal("could not open large object TOC file \"%s\" for input: %m",
+			  fname);
 
 	/* Read the blobs TOC file line-by-line, and process each blob */
 	while ((cfgets(ctx->blobsTocFH, line, MAXPGPATH)) != NULL)
@@ -463,8 +456,8 @@ _LoadBlobs(ArchiveHandle *AH)
 
 		/* Can't overflow because line and fname are the same length. */
 		if (sscanf(line, "%u %s\n", &oid, fname) != 2)
-			exit_horribly(modulename, "invalid line in large object TOC file \"%s\": \"%s\"\n",
-						  fname, line);
+			fatal("invalid line in large object TOC file \"%s\": \"%s\"",
+				  fname, line);
 
 		StartRestoreBlob(AH, oid, AH->public.ropt->dropSchema);
 		snprintf(path, MAXPGPATH, "%s/%s", ctx->directory, fname);
@@ -472,12 +465,12 @@ _LoadBlobs(ArchiveHandle *AH)
 		EndRestoreBlob(AH, oid);
 	}
 	if (!cfeof(ctx->blobsTocFH))
-		exit_horribly(modulename, "error reading large object TOC file \"%s\"\n",
-					  fname);
+		fatal("error reading large object TOC file \"%s\"",
+			  fname);
 
 	if (cfclose(ctx->blobsTocFH) != 0)
-		exit_horribly(modulename, "could not close large object TOC file \"%s\": %s\n",
-					  fname, strerror(errno));
+		fatal("could not close large object TOC file \"%s\": %m",
+			  fname);
 
 	ctx->blobsTocFH = NULL;
 
@@ -502,8 +495,8 @@ _WriteByte(ArchiveHandle *AH, const int i)
 		/* if write didn't set errno, assume problem is no disk space */
 		if (errno == 0)
 			errno = ENOSPC;
-		exit_horribly(modulename, "could not write to output file: %s\n",
-					  get_cfp_error(ctx->dataFH));
+		fatal("could not write to output file: %s",
+			  get_cfp_error(ctx->dataFH));
 	}
 
 	return 1;
@@ -538,8 +531,8 @@ _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
 		/* if write didn't set errno, assume problem is no disk space */
 		if (errno == 0)
 			errno = ENOSPC;
-		exit_horribly(modulename, "could not write to output file: %s\n",
-					  get_cfp_error(ctx->dataFH));
+		fatal("could not write to output file: %s",
+			  get_cfp_error(ctx->dataFH));
 	}
 
 	return;
@@ -560,8 +553,7 @@ _ReadBuf(ArchiveHandle *AH, void *buf, size_t len)
 	 * exit on short reads.
 	 */
 	if (cfread(buf, len, ctx->dataFH) != len)
-		exit_horribly(modulename,
-					  "could not read from input file: end of file\n");
+		fatal("could not read from input file: end of file");
 
 	return;
 }
@@ -596,8 +588,7 @@ _CloseArchive(ArchiveHandle *AH)
 		/* The TOC is always created uncompressed */
 		tocFH = cfopen_write(fname, PG_BINARY_W, 0);
 		if (tocFH == NULL)
-			exit_horribly(modulename, "could not open output file \"%s\": %s\n",
-						  fname, strerror(errno));
+			fatal("could not open output file \"%s\": %m", fname);
 		ctx->dataFH = tocFH;
 
 		/*
@@ -610,8 +601,7 @@ _CloseArchive(ArchiveHandle *AH)
 		AH->format = archDirectory;
 		WriteToc(AH);
 		if (cfclose(tocFH) != 0)
-			exit_horribly(modulename, "could not close TOC file: %s\n",
-						  strerror(errno));
+			fatal("could not close TOC file: %m");
 		WriteDataChunks(AH, ctx->pstate);
 
 		ParallelBackupEnd(AH, ctx->pstate);
@@ -621,7 +611,7 @@ _CloseArchive(ArchiveHandle *AH)
 		 * individually. Just recurse once through all the files generated.
 		 */
 		if (AH->dosync)
-			fsync_dir_recurse(ctx->directory, progname);
+			fsync_dir_recurse(ctx->directory);
 	}
 	AH->FH = NULL;
 }
@@ -661,8 +651,7 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 	/* The blob TOC file is never compressed */
 	ctx->blobsTocFH = cfopen_write(fname, "ab", 0);
 	if (ctx->blobsTocFH == NULL)
-		exit_horribly(modulename, "could not open output file \"%s\": %s\n",
-					  fname, strerror(errno));
+		fatal("could not open output file \"%s\": %m", fname);
 }
 
 /*
@@ -681,8 +670,7 @@ _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 	ctx->dataFH = cfopen_write(fname, PG_BINARY_W, AH->compression);
 
 	if (ctx->dataFH == NULL)
-		exit_horribly(modulename, "could not open output file \"%s\": %s\n",
-					  fname, strerror(errno));
+		fatal("could not open output file \"%s\": %m", fname);
 }
 
 /*
@@ -704,7 +692,7 @@ _EndBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 	/* register the blob in blobs.toc */
 	len = snprintf(buf, sizeof(buf), "%u blob_%u.dat\n", oid, oid);
 	if (cfwrite(buf, len, ctx->blobsTocFH) != len)
-		exit_horribly(modulename, "could not write to blobs TOC file\n");
+		fatal("could not write to blobs TOC file");
 }
 
 /*
@@ -736,13 +724,75 @@ setFilePath(ArchiveHandle *AH, char *buf, const char *relativeFilename)
 	dname = ctx->directory;
 
 	if (strlen(dname) + 1 + strlen(relativeFilename) + 1 > MAXPGPATH)
-		exit_horribly(modulename, "file name too long: \"%s\"\n", dname);
+		fatal("file name too long: \"%s\"", dname);
 
 	strcpy(buf, dname);
 	strcat(buf, "/");
 	strcat(buf, relativeFilename);
 }
 
+/*
+ * Prepare for parallel restore.
+ *
+ * The main thing that needs to happen here is to fill in TABLE DATA and BLOBS
+ * TOC entries' dataLength fields with appropriate values to guide the
+ * ordering of restore jobs.  The source of said data is format-dependent,
+ * as is the exact meaning of the values.
+ *
+ * A format module might also choose to do other setup here.
+ */
+static void
+_PrepParallelRestore(ArchiveHandle *AH)
+{
+	TocEntry   *te;
+
+	for (te = AH->toc->next; te != AH->toc; te = te->next)
+	{
+		lclTocEntry *tctx = (lclTocEntry *) te->formatData;
+		char		fname[MAXPGPATH];
+		struct stat st;
+
+		/*
+		 * A dumpable object has set tctx->filename, any other object has not.
+		 * (see _ArchiveEntry).
+		 */
+		if (tctx->filename == NULL)
+			continue;
+
+		/* We may ignore items not due to be restored */
+		if ((te->reqs & REQ_DATA) == 0)
+			continue;
+
+		/*
+		 * Stat the file and, if successful, put its size in dataLength.  When
+		 * using compression, the physical file size might not be a very good
+		 * guide to the amount of work involved in restoring the file, but we
+		 * only need an approximate indicator of that.
+		 */
+		setFilePath(AH, fname, tctx->filename);
+
+		if (stat(fname, &st) == 0)
+			te->dataLength = st.st_size;
+		else
+		{
+			/* It might be compressed */
+			strlcat(fname, ".gz", sizeof(fname));
+			if (stat(fname, &st) == 0)
+				te->dataLength = st.st_size;
+		}
+
+		/*
+		 * If this is the BLOBS entry, what we stat'd was blobs.toc, which
+		 * most likely is a lot smaller than the actual blob data.  We don't
+		 * have a cheap way to estimate how much smaller, but fortunately it
+		 * doesn't matter too much as long as we get the blobs processed
+		 * reasonably early.  Arbitrarily scale up by a factor of 1K.
+		 */
+		if (strcmp(te->desc, "BLOBS") == 0)
+			te->dataLength *= 1024;
+	}
+}
+
 /*
  * Clone format-specific fields during parallel restoration.
  */
