@@ -216,12 +216,6 @@ SET SESSION AUTHORIZATION regress_priv_user1;
 SELECT * FROM atest3; -- fail
 DELETE FROM atest3; -- ok
 
-BEGIN;
-RESET SESSION AUTHORIZATION;
-ALTER ROLE regress_priv_user1 NOINHERIT;
-SET SESSION AUTHORIZATION regress_priv_user1;
-DELETE FROM atest3;
-ROLLBACK;
 
 -- views
 
@@ -397,44 +391,6 @@ UPDATE t1 SET c3 = 10; -- fail, but see columns with SELECT rights, or being mod
 SET SESSION AUTHORIZATION regress_priv_user1;
 DROP TABLE t1;
 
--- check error reporting with column privs on a partitioned table
-CREATE TABLE errtst(a text, b text NOT NULL, c text, secret1 text, secret2 text) PARTITION BY LIST (a);
-CREATE TABLE errtst_part_1(secret2 text, c text, a text, b text NOT NULL, secret1 text);
-CREATE TABLE errtst_part_2(secret1 text, secret2 text, a text, c text, b text NOT NULL);
-
-ALTER TABLE errtst ATTACH PARTITION errtst_part_1 FOR VALUES IN ('aaa');
-ALTER TABLE errtst ATTACH PARTITION errtst_part_2 FOR VALUES IN ('aaaa');
-
-GRANT SELECT (a, b, c) ON TABLE errtst TO regress_priv_user2;
-GRANT UPDATE (a, b, c) ON TABLE errtst TO regress_priv_user2;
-GRANT INSERT (a, b, c) ON TABLE errtst TO regress_priv_user2;
-
-INSERT INTO errtst_part_1 (a, b, c, secret1, secret2)
-VALUES ('aaa', 'bbb', 'ccc', 'the body', 'is in the attic');
-
-SET SESSION AUTHORIZATION regress_priv_user2;
-
--- Perform a few updates that violate the NOT NULL constraint. Make sure
--- the error messages don't leak the secret fields.
-
--- simple insert.
-INSERT INTO errtst (a, b) VALUES ('aaa', NULL);
--- simple update.
-UPDATE errtst SET b = NULL;
--- partitioning key is updated, doesn't move the row.
-UPDATE errtst SET a = 'aaa', b = NULL;
--- row is moved to another partition.
-UPDATE errtst SET a = 'aaaa', b = NULL;
-
--- row is moved to another partition. This differs from the previous case in
--- that the new partition is excluded by constraint exclusion, so its
--- ResultRelInfo is not created at ExecInitModifyTable, but needs to be
--- constructed on the fly when the updated tuple is routed to it.
-UPDATE errtst SET a = 'aaaa', b = NULL WHERE a = 'aaa';
-
-SET SESSION AUTHORIZATION regress_priv_user1;
-DROP TABLE errtst;
-
 -- test column-level privileges when involved with DELETE
 SET SESSION AUTHORIZATION regress_priv_user1;
 ALTER TABLE atest6 ADD COLUMN three integer;
@@ -468,27 +424,27 @@ DELETE FROM atest5 WHERE two = 2; -- ok
 
 -- check inheritance cases
 SET SESSION AUTHORIZATION regress_priv_user1;
-CREATE TABLE atestp1 (f1 int, f2 int) WITH OIDS;
-CREATE TABLE atestp2 (fx int, fy int) WITH OIDS;
+CREATE TABLE atestp1 (f1 int, f2 int);
+CREATE TABLE atestp2 (fx int, fy int);
 CREATE TABLE atestc (fz int) INHERITS (atestp1, atestp2);
-GRANT SELECT(fx,fy,oid) ON atestp2 TO regress_priv_user2;
+GRANT SELECT(fx,fy,tableoid) ON atestp2 TO regress_priv_user2;
 GRANT SELECT(fx) ON atestc TO regress_priv_user2;
 
 SET SESSION AUTHORIZATION regress_priv_user2;
 SELECT fx FROM atestp2; -- ok
 SELECT fy FROM atestp2; -- ok
 SELECT atestp2 FROM atestp2; -- ok
-SELECT oid FROM atestp2; -- ok
+SELECT tableoid FROM atestp2; -- ok
 SELECT fy FROM atestc; -- fail
 
 SET SESSION AUTHORIZATION regress_priv_user1;
-GRANT SELECT(fy,oid) ON atestc TO regress_priv_user2;
+GRANT SELECT(fy,tableoid) ON atestc TO regress_priv_user2;
 
 SET SESSION AUTHORIZATION regress_priv_user2;
 SELECT fx FROM atestp2; -- still ok
 SELECT fy FROM atestp2; -- ok
 SELECT atestp2 FROM atestp2; -- ok
-SELECT oid FROM atestp2; -- ok
+SELECT tableoid FROM atestp2; -- ok
 
 -- privileges on functions, languages
 
@@ -556,7 +512,7 @@ GRANT ALL PRIVILEGES ON LANGUAGE sql TO PUBLIC;
 BEGIN;
 SELECT '{1}'::int4[]::int8[];
 REVOKE ALL ON FUNCTION int8(integer) FROM PUBLIC;
-SELECT '{1}'::int4[]::int8[]; --superuser, suceed
+SELECT '{1}'::int4[]::int8[]; --superuser, succeed
 SET SESSION AUTHORIZATION regress_priv_user4;
 SELECT '{1}'::int4[]::int8[]; --other user, fail
 ROLLBACK;
@@ -826,53 +782,6 @@ SELECT has_table_privilege('regress_priv_user1', 'atest4', 'SELECT WITH GRANT OP
 \c -
 CREATE ROLE regress_sro_user;
 
--- Check that index expressions and predicates are run as the table's owner
-
--- A dummy index function checking current_user
-CREATE FUNCTION sro_ifun(int) RETURNS int AS $$
-BEGIN
-	-- Below we set the table's owner to regress_sro_user
-	ASSERT current_user = 'regress_sro_user',
-		format('sro_ifun(%s) called by %s', $1, current_user);
-	RETURN $1;
-END;
-$$ LANGUAGE plpgsql IMMUTABLE;
--- Create a table owned by regress_sro_user
-CREATE TABLE sro_tab (a int);
-ALTER TABLE sro_tab OWNER TO regress_sro_user;
-INSERT INTO sro_tab VALUES (1), (2), (3);
--- Create an expression index with a predicate
-CREATE INDEX sro_idx ON sro_tab ((sro_ifun(a) + sro_ifun(0)))
-	WHERE sro_ifun(a + 10) > sro_ifun(10);
-DROP INDEX sro_idx;
--- Do the same concurrently
-CREATE INDEX CONCURRENTLY sro_idx ON sro_tab ((sro_ifun(a) + sro_ifun(0)))
-	WHERE sro_ifun(a + 10) > sro_ifun(10);
--- REINDEX
-REINDEX TABLE sro_tab;
-REINDEX INDEX sro_idx;
-REINDEX TABLE CONCURRENTLY sro_tab;  -- v12+ feature
-DROP INDEX sro_idx;
--- CLUSTER
-CREATE INDEX sro_cluster_idx ON sro_tab ((sro_ifun(a) + sro_ifun(0)));
-CLUSTER sro_tab USING sro_cluster_idx;
-DROP INDEX sro_cluster_idx;
--- BRIN index
-CREATE INDEX sro_brin ON sro_tab USING brin ((sro_ifun(a) + sro_ifun(0)));
-SELECT brin_desummarize_range('sro_brin', 0);
-SELECT brin_summarize_range('sro_brin', 0);
-DROP TABLE sro_tab;
--- Check with a partitioned table
-CREATE TABLE sro_ptab (a int) PARTITION BY RANGE (a);
-ALTER TABLE sro_ptab OWNER TO regress_sro_user;
-CREATE TABLE sro_part PARTITION OF sro_ptab FOR VALUES FROM (1) TO (10);
-ALTER TABLE sro_part OWNER TO regress_sro_user;
-INSERT INTO sro_ptab VALUES (1), (2), (3);
-CREATE INDEX sro_pidx ON sro_ptab ((sro_ifun(a) + sro_ifun(0)))
-	WHERE sro_ifun(a + 10) > sro_ifun(10);
-REINDEX TABLE sro_ptab;
-REINDEX INDEX CONCURRENTLY sro_pidx;  -- v12+ feature
-
 SET SESSION AUTHORIZATION regress_sro_user;
 CREATE FUNCTION unwanted_grant() RETURNS void LANGUAGE sql AS
 	'GRANT regress_priv_group2 TO regress_sro_user';
@@ -899,23 +808,6 @@ REFRESH MATERIALIZED VIEW sro_mv;
 REFRESH MATERIALIZED VIEW sro_mv;
 BEGIN; SET CONSTRAINTS ALL IMMEDIATE; REFRESH MATERIALIZED VIEW sro_mv; COMMIT;
 
--- REFRESH MATERIALIZED VIEW CONCURRENTLY use of eval_const_expressions()
-SET SESSION AUTHORIZATION regress_sro_user;
-CREATE FUNCTION unwanted_grant_nofail(int) RETURNS int
-	IMMUTABLE LANGUAGE plpgsql AS $$
-BEGIN
-	PERFORM unwanted_grant();
-	RAISE WARNING 'owned';
-	RETURN 1;
-EXCEPTION WHEN OTHERS THEN
-	RETURN 2;
-END$$;
-CREATE MATERIALIZED VIEW sro_index_mv AS SELECT 1 AS c;
-CREATE UNIQUE INDEX ON sro_index_mv (c) WHERE unwanted_grant_nofail(1) > 0;
-\c -
-REFRESH MATERIALIZED VIEW CONCURRENTLY sro_index_mv;
-REFRESH MATERIALIZED VIEW sro_index_mv;
-
 DROP OWNED BY regress_sro_user;
 DROP ROLE regress_sro_user;
 
@@ -1053,21 +945,6 @@ SELECT * FROM pg_largeobject LIMIT 0;
 SET SESSION AUTHORIZATION regress_priv_user1;
 SELECT * FROM pg_largeobject LIMIT 0;			-- to be denied
 
--- pg_signal_backend can't signal superusers
-RESET SESSION AUTHORIZATION;
-BEGIN;
-CREATE OR REPLACE FUNCTION terminate_nothrow(pid int) RETURNS bool
-	LANGUAGE plpgsql SECURITY DEFINER SET client_min_messages = error AS $$
-BEGIN
-	RETURN pg_terminate_backend($1);
-EXCEPTION WHEN OTHERS THEN
-	RETURN false;
-END$$;
-ALTER FUNCTION terminate_nothrow OWNER TO pg_signal_backend;
-SELECT backend_type FROM pg_stat_activity
-WHERE CASE WHEN COALESCE(usesysid, 10) = 10 THEN terminate_nothrow(pid) END;
-ROLLBACK;
-
 -- test default ACLs
 \c -
 
@@ -1078,8 +955,7 @@ CREATE TABLE testns.acltest1 (x int);
 SELECT has_table_privilege('regress_priv_user1', 'testns.acltest1', 'SELECT'); -- no
 SELECT has_table_privilege('regress_priv_user1', 'testns.acltest1', 'INSERT'); -- no
 
--- placeholder for test with duplicated schema and role names
-ALTER DEFAULT PRIVILEGES IN SCHEMA testns,testns GRANT SELECT ON TABLES TO public,public;
+ALTER DEFAULT PRIVILEGES IN SCHEMA testns GRANT SELECT ON TABLES TO public;
 
 SELECT has_table_privilege('regress_priv_user1', 'testns.acltest1', 'SELECT'); -- no
 SELECT has_table_privilege('regress_priv_user1', 'testns.acltest1', 'INSERT'); -- no
@@ -1249,9 +1125,7 @@ SELECT has_function_privilege('regress_priv_user1', 'testns.priv_testfunc(int)',
 SELECT has_function_privilege('regress_priv_user1', 'testns.priv_testagg(int)', 'EXECUTE'); -- true
 SELECT has_function_privilege('regress_priv_user1', 'testns.priv_testproc(int)', 'EXECUTE'); -- true
 
-\set VERBOSITY terse \\ -- suppress cascade details
 DROP SCHEMA testns CASCADE;
-\set VERBOSITY default
 
 
 -- Change owner of the schema & and rename of new schema owner
@@ -1270,9 +1144,7 @@ ALTER ROLE regress_schemauser2 RENAME TO regress_schemauser_renamed;
 SELECT nspname, rolname FROM pg_namespace, pg_roles WHERE pg_namespace.nspname = 'testns' AND pg_namespace.nspowner = pg_roles.oid;
 
 set session role regress_schemauser_renamed;
-\set VERBOSITY terse \\ -- suppress cascade details
 DROP SCHEMA testns CASCADE;
-\set VERBOSITY default
 
 -- clean up
 \c -
